program KeyboardHandler;
{****************************************************************************
** Demonstration of using a custom keyboard handler to detect multiple     **
** cursor keys being held down simultaneously                              **
**  by Steven H Don                                                        **
**                                                                         **
** For questions, feel free to e-mail me.                                  **
**                                                                         **
**    shd@earthling.net                                                    **
**    http://shd.cjb.net                                                   **
**                                                                         **
****************************************************************************}

uses Dos, Crt;

var
  {Key status indicators}
  CursorUp,
  CursorDown,
  CursorLeft,
  CursorRight : Boolean;

  {Pointer to original keyboard handler}
  BIOSHandler : procedure;

{****************************************************************************
** This is the custom keyboard handler. It checks to see if a cursor key   **
** is pressed or release and sets variables accordingly.                   **
****************************************************************************}

{$F+}
procedure KeyCheck; interrupt;
var
  CallBIOSHandler : Boolean;

begin
  {Assume BIOS has to be called}
  CallBIOSHandler := true;
  case port [$60] of
    {Up key pressed}
    72 : begin
      {Set up indicator}
      CursorUp := true;
      {No need to call BIOS}
      CallBIOSHandler := false;
    end;
    {Up key depressed}
    200 : begin
      {Reset up indicator}
      CursorUp := false;
      {No need to call BIOS}
      CallBIOSHandler := false;
    end;
    {Left key pressed}
    75 : begin
      {Set left indicator}
      CursorLeft := true;
      {No need to call BIOS}
      CallBIOSHandler := false;
    end;
    {Left key depressed}
    203 : begin
      {Reset left indicator}
      CursorLeft := false;
      {No need to call BIOS}
      CallBIOSHandler := false;
    end;
    {Right key pressed}
    77 : begin
      {Set right indicator}
      CursorRight := true;
      {No need to call BIOS}
      CallBIOSHandler := false;
    end;
    {Right key depressed}
    205 : begin
      {Reset right indicator}
      CursorRight := false;
      {No need to call BIOS}
      CallBIOSHandler := false;
    end;
    {Down key pressed}
    80 : begin
      {Set down indicator}
      CursorDown := true;
      {No need to call BIOS}
      CallBIOSHandler := false;
    end;
    {Down key depressed}
    208 : begin
      {Reset down indicator}
      CursorDown := false;
      {No need to call BIOS}
      CallBIOSHandler := false;
    end;
  end;
  {If we haven't handled the keypress ourselves...}
  if CallBIOSHandler then begin
    {...let the BIOS take care of it...}
    asm pushf end;
    BIOSHandler;
  end else begin
    {...otherwise, acknowledge the hardware interrupt}
    port [$20] := $20;
  end;
end;
{$F-}

{****************************************************************************
** This procedures installs the custom keyboard handler and saves a        **
** pointer to the original                                                 **
****************************************************************************}
procedure XKey;
begin
  {Save BIOS keyboard handler}
  GetIntVec (9, @BIOSHandler);
  {Set interrupt vector to our own routine}
  SetIntVec (9, Addr(KeyCheck));
end;

{****************************************************************************
** This procedures restores the original keyboard handler                  **
*****************************************************************************
** Make sure you call this before exiting your program! Otherwise, the     **
** system will crash                                                       **
****************************************************************************}
procedure NKey;
begin
  {Set interrupt vector to the BIOS keyboard handler}
  SetIntVec (9, @BIOSHandler);
end;

begin
  {Install eXtended KEYboardhandler}
  XKey;

  {Clear the screen}
  ClrScr;

  {Show message}
  Writeln ('Demonstration of custom keyboard handler');
  repeat
    {Show cursor key status}
    GotoXY (1, 3);
    Writeln (' Up    : ', CursorUp   , ' ');
    Writeln (' Down  : ', CursorDown , ' ');
    Writeln (' Left  : ', CursorLeft , ' ');
    Writeln (' Right : ', CursorRight, ' ');
  until KeyPressed and (ReadKey = Chr(27));

  {Set Normal KEYboardhandler}
  NKey;
end.